/* copyright (c) 2007 magnus auvinen, see licence.txt for more info */
#include <engine/e_server_interface.h>
#include <game/server/entities/character.hpp>
#include <game/server/player.hpp>
#include <game/server/gamecontext.hpp>
#include "tdm.hpp"

GAMECONTROLLER_TDM::GAMECONTROLLER_TDM()
{
	gametype = "JUG";
	game_flags = GAMEFLAG_TEAMS;
	new_jug_tick = -1;
}

int GAMECONTROLLER_TDM::on_character_death(class PLAYER *victim, class PLAYER *killer, int weapon)
{
	GAMECONTROLLER::on_character_death(victim, killer, weapon);
	
	if(weapon != WEAPON_GAME && weapon != WEAPON_SELF && weapon != WEAPON_WORLD)
	{
		if(killer && victim)
		{
			if(killer->team == 0 && victim->team && killer->get_character())
			{
				victim->team = 0;
				game.controller->on_player_info_change(victim);

				new_jug = killer;
				new_jug_tick = server_tick();
				new_jug_countdown = 0;
			}
		}
	}
		
	return 0;
}

void GAMECONTROLLER_TDM::tick()
{
	int jug = 0;
	int players = 0;

	for(int i = 0; i < MAX_CLIENTS; i++)
	{
		if(game.players[i])
		{
			if(game.players[i]->team == 1)
				jug++;
			else if(game.players[i]->team == 0)
				players++;
		}
	}

	if(jug == 0 && players > 0)
	{
		if(new_jug_tick == -1)
		{
			for(int i = 0; i < MAX_CLIENTS; i++)
			{
				if(game.players[i])
				{
					if(game.players[i]->get_character() && game.players[i]->team == 0)
					{
						new_jug = game.players[i];
						new_jug_tick = server_tick() + server_tickspeed() * 2.1f;
						new_jug_countdown = 0;
						break;
					}
				}
			}
		}

		if(server_tick() >= new_jug_tick + server_tickspeed() * 0.7 * new_jug_countdown && new_jug_tick != -1)
		{
			if(new_jug_countdown == 3)
			{
				new_jug_tick = -1;

				if(new_jug)
				{
					if(new_jug->get_character() && new_jug->team == 0)
					{
						new_jug->set_team(1);

						new_jug->get_character()->real_health = 50;

						char buf[512];
						str_format(buf, sizeof(buf), "%s is the new juggernaut!", server_clientname(new_jug->client_id));
						game.create_sound_global(SOUND_CTF_CAPTURE);
						game.send_chat(-1, GAMECONTEXT::CHAT_ALL, buf); 
						game.send_broadcast(buf, -1);	
					}
				}
			}
			else
			{
				char buf[128];
				str_format(buf, sizeof(buf), "%d", 3 - new_jug_countdown);

				game.create_sound_global(SOUND_WEAPON_NOAMMO);
				game.send_broadcast(buf, -1);
			}

			new_jug_countdown++;
		}
	}

	do_player_score_wincheck();
	GAMECONTROLLER::tick();
}
